@@ -34,61 +34,62 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "library.h"
-#include "global.h"
+#include "alloc.h"
 
-static char const rcsid[] = "$Id: alloc.c,v 1.5 2002/07/28 15:40:07 broeker Exp $";
+#include "global.h" /* for postfatal() */
+
+static char const rcsid[] = "$Id: alloc.c,v 1.9 2014/11/20 21:12:54 broeker Exp $";
 
 static	void	*alloctest(void *p);
 
 /* let autoconf find out if <stdlib.h> is available. This test will
  * succeed more reliably than the defined(__STDC__) one I replaced */
 #if STDC_HEADERS
-#include <stdlib.h>
-# else
+# include <stdlib.h>
+#else
 char	*calloc(), *malloc(), *realloc(), *strcpy();
-# endif
+#endif
 
 /* allocate a string */
 
 char *
-stralloc(char *s)
+my_strdup(char *s)
 {
-	return(strcpy(mymalloc((int) strlen(s) + 1), s));
+	return(strcpy(mymalloc(strlen(s) + 1), s));
 }
 
+
 /* version of malloc that only returns if successful */
-
 void *
-mymalloc(int size)
+mymalloc(size_t size)
 {
-	return(alloctest(malloc((unsigned) size)));
+    return(alloctest(malloc(size)));
 }
 
+
 /* version of calloc that only returns if successful */
-
 void *
-mycalloc(int nelem, int size)
+mycalloc(size_t nelem, size_t size)
 {
-	return(alloctest(calloc((unsigned) nelem, (unsigned) size)));
+    return(alloctest(calloc(nelem, size)));
 }
 
+
 /* version of realloc that only returns if successful */
-
 void *
-myrealloc(void *p, int size)
+myrealloc(void *p, size_t size)
 {
-	return(alloctest(realloc(p, (unsigned) size)));
+    return(alloctest(realloc(p, size)));
 }
 
-/* check for memory allocation failure */
 
+/* check for memory allocation failure */
 static	void *
 alloctest(void *p)
 {
-	if (p == NULL) {
-		(void) fprintf(stderr, "\n%s: out of storage\n", argv0);
-		myexit(1);
-	}
-	return(p);
+    if (p == NULL) {
+	postfatal("\n%s: out of storage\n", argv0);
+	/* NOTREACHED */
+    }
+    return(p);
 }
