@@ -47,7 +47,7 @@
 #endif
 #include <ctype.h>
 
-static char const rcsid[] = "$Id: command.c,v 1.32 2006/08/20 15:00:33 broeker Exp $";
+static char const rcsid[] = "$Id: command.c,v 1.37 2014/11/20 21:12:54 broeker Exp $";
 
 
 int	selecting;
@@ -314,9 +314,7 @@ command(int commandc)
 		s = "a";
 	    }
 	    if (c != '\r' && 
-		mygetline("", newpat,
-			  COLS - sizeof(appendprompt), c, NO) > 0
-		) {
+		mygetline("", newpat, COLS - sizeof(appendprompt), c, NO) > 0) {
 		shellpath(filename, sizeof(filename), newpat);
 		if ((file = myfopen(filename, s)) == NULL) {
 		    cannotopen(filename);
@@ -336,8 +334,7 @@ command(int commandc)
     case '<':	/* read lines from a file */
 	move(PRLINE, 0);
 	addstr(readprompt);
-	if (mygetline("", newpat, COLS - sizeof(readprompt),
-		      '\0', NO) > 0) {
+	if (mygetline("", newpat, COLS - sizeof(readprompt), '\0', NO) > 0) {
 	    clearprompt();
 	    shellpath(filename, sizeof(filename), newpat);
 	    if (readrefs(filename) == NO) {
@@ -358,8 +355,7 @@ command(int commandc)
 	/* get the shell command */
 	move(PRLINE, 0);
 	addstr(pipeprompt);
-	if (mygetline("", newpat, COLS - sizeof(pipeprompt), '\0', NO)
-	    == 0) {
+	if (mygetline("", newpat, COLS - sizeof(pipeprompt), '\0', NO) == 0) {
 	    clearprompt();
 	    return(NO);
 	}
@@ -449,8 +445,9 @@ cscope: cannot open pipe to shell command: %s\n", newpat);
 
     case ctrl('B'):		/* cmd history back */
     case ctrl('F'):		/* cmd history fwd */
-	if (selecting)
-	    return(NO);
+	if (selecting) {
+	    selecting = 0;
+	}
 
 	curritem = currentcmd();
 	item = (commandc == ctrl('F')) ? nextcmd() : prevcmd();
@@ -502,7 +499,7 @@ cscope: cannot open pipe to shell command: %s\n", newpat);
 	/* FALLTHROUGH */
     default:
 	if (selecting && !mouse) {
-	    char		*c;
+	    char *c;
 
 	    if ((c = strchr(dispchars, commandc)))
 		editref(c - dispchars);
@@ -622,7 +619,7 @@ changestring(void)
 	return(NO);
     }
     /* create the line change indicators */
-    change = mycalloc(totallines, sizeof(BOOL));
+    change = mycalloc(totallines, sizeof(*change));
     changing = YES;
     mousemenu();
 
@@ -634,11 +631,17 @@ changestring(void)
 	atchange();
 		
 	/* get a character from the terminal */
-	if ((c = mygetch()) == EOF
-	    || c == ctrl('D') 
-	    || c == ctrl('Z')) {
+	if ((c = mygetch()) == EOF || c == ctrl('D')) {
 	    break;	/* change lines */
 	}
+	if (c == ctrl('Z')) {
+#ifdef SIGTSTP
+	    kill(0, SIGTSTP);
+	    goto same;
+#else
+	    break;	/* change lines */
+#endif
+	}
 	/* see if the input character is a command */
 	switch (c) {
 	case ' ':	/* display next page */
@@ -739,7 +742,7 @@ changestring(void)
 				
 		/* make sure it can be changed */
 		if (access(newfile, WRITE) != 0) {
-		    sprintf(msg, "Cannot write to file %s", newfile);
+		    snprintf(msg, sizeof(msg), "Cannot write to file %s", newfile);
 		    postmsg(msg);
 		    anymarked = NO;
 		    break;
@@ -884,12 +887,20 @@ countrefs(void)
     filelen = 4;		/* strlen("File") */
     fcnlen = 8;		/* strlen("Function") */
     numlen = 0;
-    while ((i = fscanf(refsfound, "%250s%250s%5s %5000[^\n]", file,
-		       function, linenum, tempstring)) != EOF) {
-	if (i != 4 ||
-	    !isgraph((unsigned char) *file) ||
-	    !isgraph((unsigned char) *function) ||
-	    !isdigit((unsigned char) *linenum)) {
+    /* HBB NOTE 2012-04-07: it may look like we shouldn't assing tempstring here,
+     * since it's not used.  But it has to be assigned just so the return value
+     * of fscanf will actually reach 4. */
+    while (EOF != (i = fscanf(refsfound, 
+			      "%" PATHLEN_STR "s%" PATLEN_STR "s%" NUMLEN_STR "s %" TEMPSTRING_LEN_STR "[^\n]",
+			      file, function, linenum, tempstring
+			     )
+	          )
+	  ) {
+	if (   (i != 4)
+	    || !isgraph((unsigned char) *file)
+	    || !isgraph((unsigned char) *function)
+	    || !isdigit((unsigned char) *linenum)
+	   ) {
 	    postmsg("File does not have expected format");
 	    totallines = 0;
 	    disprefs = 0;
