@@ -37,12 +37,14 @@
 
 #include "global.h"
 #include "build.h"
+#include "alloc.h"
 
 #ifdef CCS
 #include "sgs.h"	/* ESG_PKG and ESG_REL */
 #else
 #include "version.h"	/* FILEVERSION and FIXVERSION */
 #endif
+
 #if defined(USE_NCURSES) && !defined(RENAMED_NCURSES)
 #include <ncurses.h>
 #else
@@ -54,29 +56,35 @@
 #include <errno.h>
 #include <stdarg.h>
 
-static char const rcsid[] = "$Id: display.c,v 1.22 2003/06/12 17:11:38 broeker Exp $";
+#ifndef HAVE_SIGSETJMP
+# define sigsetjmp(a,b) setjmp(a)
+# define siglongjmp(a,b) longjmp(a,b)
+typedef jmp_buf sigjmp_buf;
+#endif
+
+static char const rcsid[] = "$Id: display.c,v 1.33 2012/05/20 12:24:17 broeker Exp $";
 
 int	booklen;		/* OGS book name display field length */
 int	*displine;		/* screen line of displayed reference */
-int	disprefs;		/* displayed references */
+unsigned int disprefs;		/* displayed references */
 int	field;			/* input field */
 int	filelen;		/* file name display field length */
 int	fcnlen;			/* function name display field length */
-int	mdisprefs;		/* maximum displayed references */
-int	nextline;		/* next line to be shown */
+unsigned int mdisprefs;		/* maximum displayed references */
+unsigned int nextline;		/* next line to be shown */
 FILE	*nonglobalrefs;		/* non-global references file */
 int	numlen;			/* line number display field length */
-int	topline = 1;		/* top line of page */
+unsigned int topline = 1;		/* top line of page */
 int	bottomline;		/* bottom line of page */
 long	searchcount;		/* count of files searched */
 int	subsystemlen;		/* OGS subsystem name display field length */
-int	totallines;		/* total reference lines */
+unsigned int totallines;	/* total reference lines */
 unsigned fldcolumn;		/* input field column */
 
 const char	dispchars[] = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 static	int	fldline;		/* input field line */
-static	jmp_buf	env;			/* setjmp/longjmp buffer */
+static	sigjmp_buf	env;		/* setjmp/longjmp buffer */
 static	int	lastdispline;		/* last displayed reference line */
 static	char	lastmsg[MSGLEN + 1];	/* last message displayed */
 static	char	helpstring[] = "Press the ? key for help";
@@ -104,6 +112,7 @@ static	struct	{		/* text of input fields */
 	{"Find this", "egrep pattern",			findregexp},
 	{"Find this", "file",				findfile},
 	{"Find", "files #including this file",		findinclude},
+	{"Find", "assignments to this symbol", 		findassign},
 	{"Find all", "function definitions",		findallfcns},	/* samuel only */
 };
 
@@ -121,8 +130,8 @@ dispinit(void)
 
 
 	if (mdisprefs <= 0) {
-		(void) fprintf(stderr, "%s: screen too small\n", argv0);
-		myexit(1);
+		postfatal("%s: screen too small\n", argv0);
+		/* NOTREACHED */
 	}
 
 	if (mouse == NO && mdisprefs > strlen(dispchars))
@@ -137,233 +146,229 @@ dispinit(void)
 void
 display(void)
 {
-	char	*subsystem;		/* OGS subsystem name */
-	char	*book;			/* OGS book name */
-	char	file[PATHLEN + 1];	/* file name */
-	char	function[PATLEN + 1];	/* function name */
-	char	linenum[NUMLEN + 1];	/* line number */
-	int	screenline;		/* screen line number */
-	int	width;			/* source line display width */
-	int	i;
-	char	*s;
+    char    *subsystem;             /* OGS subsystem name */
+    char    *book;                  /* OGS book name */
+    char    file[PATHLEN + 1];      /* file name */
+    char    function[PATLEN + 1];   /* function name */
+    char    linenum[NUMLEN + 1];    /* line number */
+    int     screenline;             /* screen line number */
+    int     width;                  /* source line display width */
+    int     i;
+    char    *s;
 
-	/* see if this is the initial display */
-	erase();
-	if (refsfound == NULL) {
+    /* see if this is the initial display */
+    erase();
+    if (refsfound == NULL) {
 #if CCS
-		if (displayversion == YES) {
-			printw("cscope %s", ESG_REL);
-		}
-		else {
-			printw("cscope");
-		}
+	if (displayversion == YES) {
+	    printw("cscope %s", ESG_REL);
+	}
+	else {
+	    printw("cscope");
+	}
 #else
-		printw("Cscope version %d%s", FILEVERSION, FIXVERSION);
+	printw("Cscope version %d%s", FILEVERSION, FIXVERSION);
 #endif
-		move(0, COLS - (int) sizeof(helpstring));
-		addstr(helpstring);
-	} else if (totallines == 0) {
-		/* if no references were found */
-		/* redisplay the last message */
-		addstr(lastmsg);
-	} else {
-		/* display the pattern */
-		if (changing == YES) {
-			printw("Change \"%s\" to \"%s\"", pattern, newpat);
-		} else {
-			printw("%c%s: %s", toupper((unsigned char)fields[field].text2[0]),
-				fields[field].text2 + 1, pattern);
-		}
-		/* display the column headings */
-		move(2, 2);
-		if (ogs == YES && field != FILENAME) {
-			printw("%-*s ", subsystemlen, "Subsystem");
-			printw("%-*s ", booklen, "Book");
-		}
-		if (dispcomponents > 0)
-			printw("%-*s ", filelen, "File");
-
-		if (field == SYMBOL || field == CALLEDBY || field == CALLING) {
-			printw("%-*s ", fcnlen, "Function");
-		}
-		if (field != FILENAME) {
-			addstr("Line");
-		}
-		addch('\n');
-
-		/* if at end of file go back to beginning */
-		if (nextline > totallines) {
-			seekline(1);
-		}
-		/* calculate the source text column */
-
-		width = COLS - numlen - 3;
-
-		if (ogs == YES) {
-			width -= subsystemlen + booklen + 2;
-		}
-		if (dispcomponents > 0) {
-			width -= filelen + 1;
-		}
-		if (field == SYMBOL || field == CALLEDBY || field == CALLING) {
-			width -= fcnlen + 1;
-		}
-
-		/* until the max references have been displayed or 
-		   there is no more room */
-		topline = nextline;
-		for (disprefs = 0, screenline = REFLINE;
-		     disprefs < mdisprefs && screenline <= lastdispline;
-		     ++disprefs, ++screenline) {
-			/* read the reference line */
-			if (fscanf(refsfound, "%s%s%s %[^\n]", file, function, 
-			    linenum, tempstring) < 4) {
-				break;
-			}
-			++nextline;
-			displine[disprefs] = screenline;
-			
-			/* if no mouse, display the selection number */
-			if (mouse == YES) {
-				addch(' ');
-			} else {
-				printw("%c", dispchars[disprefs]);
-			}
-
-			/* display any change mark */
-			if (changing == YES && 
-			    change[topline + disprefs - 1] == YES) {
-				addch('>');
-			} else {
-				addch(' ');
-			}
-
-			/* display the file name */
-			if (field == FILENAME) {
-				printw("%-*s ", filelen, file);
-			} else {
-				/* if OGS, display the subsystem and book names */
-				if (ogs == YES) {
-					ogsnames(file, &subsystem, &book);
-					printw("%-*.*s ", subsystemlen, subsystemlen, subsystem);
-					printw("%-*.*s ", booklen, booklen, book);
-				}
-				/* display the requested path components */
-				if (dispcomponents > 0) {
-					printw("%-*.*s ", filelen, filelen,
-						pathcomponents(file, dispcomponents));
-				}
-			} /* else(field == FILENAME) */
-
-			/* display the function name */
-			if (field == SYMBOL || field == CALLEDBY || field == CALLING) {
-				printw("%-*.*s ", fcnlen, fcnlen, function);
-			}
-			if (field == FILENAME) {
-				addch('\n');	/* go to next line */
-				continue;
-			}
-
-			/* display the line number */
-			printw("%*s ", numlen, linenum);
-			/* there may be tabs in egrep output */
-			while ((s = strchr(tempstring, '\t')) != NULL) {
-				*s = ' ';
-			}
-
-			/* display the source line */
-			s = tempstring;
-			for (;;) {
-				/* see if the source line will fit */
-				if ((i = strlen(s)) > width) {
-					
-					/* find the nearest blank */
-					for (i = width; s[i] != ' ' && i > 0; --i) {
-						;
-					}
-					if (i == 0) {
-						i = width;	/* no blank */
-					}
-				}
-				/* print up to this point */
-				printw("%.*s", i, s);
-				s += i;
-				
-				/* if line didn't wrap around */
-				if (i < width) {
-					addch('\n');	/* go to next line */
-				}
-				/* skip blanks */
-				while (*s == ' ') {
-					++s;
-				}
-				/* see if there is more text */
-				if (*s == '\0') {
-					break;
-				}
-				/* if the source line is too long */
-				if (++screenline > lastdispline) {
-
-					/* if this is the first displayed line,
-					   display what will fit on the screen */
-					if (topline == nextline -1) {
-						disprefs++;
-						/* break out of two loops */
-						goto endrefs;
-					}
-					
-					/* erase the reference */
-					while (--screenline >= displine[disprefs]) {
-						move(screenline, 0);
-						clrtoeol();
-					}
-					++screenline;
-					 
-					/* go back to the beginning of this reference */
-					--nextline;
-					seekline(nextline);
-					goto endrefs;
-				}
-				/* indent the continued source line */
-				move(screenline, COLS - width);
-			} /* for(ever) */
-		} /* for(reference output lines) */
-	endrefs:
-		/* position the cursor for the message */
-		i = FLDLINE - 1;
-		if (screenline < i) {
-			addch('\n');
-		}
-		else {
-			move(i, 0);
-		}
-		/* check for more references */
-		i = totallines - nextline + 1;
-		bottomline = nextline;
-		if (i > 0) {
-			s = "s";
-			if (i == 1) {
-				s = "";
-			}
-			printw("* %d more line%s - press the space bar to display more *", i, s);
-		}
-		/* if this is the last page of references */
-		else if (topline > 1 && nextline > totallines) {
-			addstr("* Press the space bar to display the first lines again *");
-		}
-	}
-	/* display the input fields */
-	move(FLDLINE, 0);
-	for (i = 0; i < FIELDS; ++i) {
-		printw("%s %s:\n", fields[i].text1, fields[i].text2);
-	}
-	/* display any prompt */
+	move(0, COLS - (int) sizeof(helpstring));
+	addstr(helpstring);
+    } else if (totallines == 0) {
+	/* if no references were found */
+	/* redisplay the last message */
+	addstr(lastmsg);
+    } else {
+	/* display the pattern */
 	if (changing == YES) {
-		move(PRLINE, 0);
-		addstr(selprompt);
+	    printw("Change \"%s\" to \"%s\"", Pattern, newpat);
+	} else {
+	    printw("%c%s: %s", toupper((unsigned char)fields[field].text2[0]),
+		   fields[field].text2 + 1, Pattern);
 	}
-	drawscrollbar(topline, nextline);	/* display the scrollbar */
-	refresh();
+	/* display the column headings */
+	move(2, 2);
+	if (ogs == YES && field != FILENAME) {
+	    printw("%-*s ", subsystemlen, "Subsystem");
+	    printw("%-*s ", booklen, "Book");
+	}
+	if (dispcomponents > 0)
+	    printw("%-*s ", filelen, "File");
+
+	if (field == SYMBOL || field == CALLEDBY || field == CALLING) {
+	    printw("%-*s ", fcnlen, "Function");
+	}
+	if (field != FILENAME) {
+	    addstr("Line");
+	}
+	addch('\n');
+
+	/* if at end of file go back to beginning */
+	if (nextline > totallines) {
+	    seekline(1);
+	}
+	/* calculate the source text column */
+
+	width = COLS - numlen - 3;
+
+	if (ogs == YES) {
+	    width -= subsystemlen + booklen + 2;
+	}
+	if (dispcomponents > 0) {
+	    width -= filelen + 1;
+	}
+	if (field == SYMBOL || field == CALLEDBY || field == CALLING) {
+	    width -= fcnlen + 1;
+	}
+
+	/* until the max references have been displayed or 
+	   there is no more room */
+	topline = nextline;
+	for (disprefs = 0, screenline = REFLINE;
+	     disprefs < mdisprefs && screenline <= lastdispline;
+	     ++disprefs, ++screenline) {
+	    /* read the reference line */
+	    if (fscanf(refsfound, "%" PATHLEN_STR "s%" PATHLEN_STR "s%" NUMLEN_STR "s %" TEMPSTRING_LEN_STR "[^\n]", file, function, 
+		       linenum, tempstring) < 4) {
+		break;
+	    }
+	    ++nextline;
+	    displine[disprefs] = screenline;
+			
+	    /* if no mouse, display the selection number */
+	    if (mouse == YES) {
+		addch(' ');
+	    } else {
+		printw("%c", dispchars[disprefs]);
+	    }
+
+	    /* display any change mark */
+	    if (changing == YES && 
+		change[topline + disprefs - 1] == YES) {
+		addch('>');
+	    } else {
+		addch(' ');
+	    }
+
+	    /* display the file name */
+	    if (field == FILENAME) {
+		printw("%-*s ", filelen, file);
+	    } else {
+		/* if OGS, display the subsystem and book names */
+		if (ogs == YES) {
+		    ogsnames(file, &subsystem, &book);
+		    printw("%-*.*s ", subsystemlen, subsystemlen, subsystem);
+		    printw("%-*.*s ", booklen, booklen, book);
+		}
+		/* display the requested path components */
+		if (dispcomponents > 0) {
+		    printw("%-*.*s ", filelen, filelen,
+			   pathcomponents(file, dispcomponents));
+		}
+	    } /* else(field == FILENAME) */
+
+	    /* display the function name */
+	    if (field == SYMBOL || field == CALLEDBY || field == CALLING) {
+		printw("%-*.*s ", fcnlen, fcnlen, function);
+	    }
+	    if (field == FILENAME) {
+		addch('\n');	/* go to next line */
+		continue;
+	    }
+
+	    /* display the line number */
+	    printw("%*s ", numlen, linenum);
+	    /* there may be tabs in egrep output */
+	    while ((s = strchr(tempstring, '\t')) != NULL) {
+		*s = ' ';
+	    }
+
+	    /* display the source line */
+	    s = tempstring;
+	    for (;;) {
+		/* see if the source line will fit */
+		if ((i = strlen(s)) > width) {
+					
+		    /* find the nearest blank */
+		    for (i = width; s[i] != ' ' && i > 0; --i) {
+			;
+		    }
+		    if (i == 0) {
+			i = width;	/* no blank */
+		    }
+		}
+		/* print up to this point */
+		printw("%.*s", i, s);
+		s += i;
+				
+		/* if line didn't wrap around */
+		if (i < width) {
+		    addch('\n');	/* go to next line */
+		}
+		/* skip blanks */
+		while (*s == ' ') {
+		    ++s;
+		}
+		/* see if there is more text */
+		if (*s == '\0') {
+		    break;
+		}
+		/* if the source line is too long */
+		if (++screenline > lastdispline) {
+
+		    /* if this is the first displayed line,
+		       display what will fit on the screen */
+		    if (topline == nextline -1) {
+			disprefs++;
+			/* break out of two loops */
+			goto endrefs;
+		    }
+					
+		    /* erase the reference */
+		    while (--screenline >= displine[disprefs]) {
+			move(screenline, 0);
+			clrtoeol();
+		    }
+		    ++screenline;
+					 
+		    /* go back to the beginning of this reference */
+		    --nextline;
+		    seekline(nextline);
+		    goto endrefs;
+		}
+		/* indent the continued source line */
+		move(screenline, COLS - width);
+	    } /* for(ever) */
+	} /* for(reference output lines) */
+    endrefs:
+	/* position the cursor for the message */
+	i = FLDLINE - 1;
+	if (screenline < i) {
+	    addch('\n');
+	}
+	else {
+	    move(i, 0);
+	}
+	/* check for more references */
+	i = totallines - nextline + 1;
+	bottomline = nextline;
+	if (i > 0) {
+	    printw("* Lines %d-%d of %d, %d more - press the space bar to display more *", topline, bottomline, totallines, i);
+	}
+	/* if this is the last page of references */
+	else if (topline > 1 && nextline > totallines) {
+	    addstr("* Press the space bar to display the first lines again *");
+	}
+    }
+    /* display the input fields */
+    move(FLDLINE, 0);
+    for (i = 0; i < FIELDS; ++i) {
+	printw("%s %s:\n", fields[i].text1, fields[i].text2);
+    }
+    /* display any prompt */
+    if (changing == YES) {
+	move(PRLINE, 0);
+	addstr(selprompt);
+    }
+    drawscrollbar(topline, nextline);	/* display the scrollbar */
+    refresh();
 }
 
 /* set the cursor position for the field */
@@ -396,24 +401,21 @@ atchange(void)
 static RETSIGTYPE
 jumpback(int sig)
 {
-	(void) sig;		/* 'use' sig, to avoid warning from compiler */
-	longjmp(env, 1);
+	/* HBB NEW 20031008: try whether reinstating signal handler
+	 * helps... */
+	signal(sig, jumpback);
+	siglongjmp(env, 1);
 }
 
 BOOL
 search(void)
 {
-	char	*subsystem;		/* OGS subsystem name */
-	char 	*book;			/* OGS book name */
-	char	file[PATHLEN + 1];	/* file name */
-	char	function[PATLEN + 1];	/* function name */
-	char	linenum[NUMLEN + 1];	/* line number */
 	char	*findresult = NULL;	/* find function output */
 	BOOL	funcexist = YES;		/* find "function" error */
 	FINDINIT rc = NOERROR;		/* findinit return code */
-	RETSIGTYPE	(*savesig)(int);		/* old value of signal */
+	sighandler_t savesig;		/* old value of signal */
 	FP	f;			/* searching function */
-	int	c, i;
+	int	c;
 	
 	/* open the references found file for writing */
 	if (writerefsfound() == NO) {
@@ -424,29 +426,29 @@ search(void)
 		postmsg("Searching");
 	}
 	searchcount = 0;
-	if (setjmp(env) == 0) {
-		savesig = signal(SIGINT, jumpback);
+	savesig = signal(SIGINT, jumpback);
+	if (sigsetjmp(env, 1) == 0) {
 		f = fields[field].findfcn;
 		if (f == findregexp || f == findstring) {
-			findresult = (*f)(pattern);
-		}
-		else {
+			findresult = (*f)(Pattern);
+		} else {
 			if ((nonglobalrefs = myfopen(temp2, "wb")) == NULL) {
 				cannotopen(temp2);
 				return(NO);
 			}
-			if ((rc = findinit(pattern)) == NOERROR) {
+			if ((rc = findinit(Pattern)) == NOERROR) {
 				(void) dbseek(0L); /* read the first block */
-				findresult = (*f)(pattern);
+				findresult = (*f)(Pattern);
 				if (f == findcalledby) 
 					funcexist = (*findresult == 'y');
 				findcleanup();
 
 				/* append the non-global references */
 				(void) fclose(nonglobalrefs);
-				if ( (nonglobalrefs = myfopen(temp2, "rb")) == NULL) {
-				  cannotopen(temp2);
-				  return(NO);
+				if ((nonglobalrefs = myfopen(temp2, "rb"))
+				     == NULL) {
+					cannotopen(temp2);
+					return(NO);
 				}
 				while ((c = getc(nonglobalrefs)) != EOF) {
 					(void) putc(c, refsfound);
@@ -462,7 +464,7 @@ search(void)
 	
 	/* reopen the references found file for reading */
 	(void) fclose(refsfound);
-	if ( (refsfound = myfopen(temp1, "rb")) == NULL) {
+	if ((refsfound = myfopen(temp1, "rb")) == NULL) {
 		cannotopen(temp1);
 		return(NO);
 	}
@@ -473,64 +475,31 @@ search(void)
 	/* see if it is empty */
 	if ((c = getc(refsfound)) == EOF) {
 		if (findresult != NULL) {
-			(void) sprintf(lastmsg, "Egrep %s in this pattern: %s", 
-				findresult, pattern);
-		}
-		else if (rc == NOTSYMBOL) {
-			(void) sprintf(lastmsg, "This is not a C symbol: %s", 
-				pattern);
-		}
-		else if (rc == REGCMPERROR) {
-			(void) sprintf(lastmsg, "Error in this regcomp(3) regular expression: %s", 
-				pattern);
+			(void) snprintf(lastmsg, sizeof(lastmsg), "Egrep %s in this pattern: %s", 
+				       findresult, Pattern);
+		} else if (rc == NOTSYMBOL) {
+			(void) snprintf(lastmsg, sizeof(lastmsg), "This is not a C symbol: %s", 
+				       Pattern);
+		} else if (rc == REGCMPERROR) {
+			(void) snprintf(lastmsg, sizeof(lastmsg), "Error in this regcomp(3) regular expression: %s", 
+				       Pattern);
 			
-		}
-		else if (funcexist == NO) {
-			(void) sprintf(lastmsg, "Function definition does not exist: %s", 
-				pattern);
-		}
-		else {
-			(void) sprintf(lastmsg, "Could not find the %s: %s", 
-				fields[field].text2, pattern);
+		} else if (funcexist == NO) {
+			(void) snprintf(lastmsg, sizeof(lastmsg), "Function definition does not exist: %s", 
+				       Pattern);
+		} else {
+			(void) snprintf(lastmsg, sizeof(lastmsg), "Could not find the %s: %s", 
+				       fields[field].text2, Pattern);
 		}
 		return(NO);
 	}
 	/* put back the character read */
 	(void) ungetc(c, refsfound);
 
-	/* count the references found and find the length of the file,
-	   function, and line number display fields */
-	subsystemlen = 9;	/* strlen("Subsystem") */
-	booklen = 4;		/* strlen("Book") */
-	filelen = 4;		/* strlen("File") */
-	fcnlen = 8;		/* strlen("Function") */
-	numlen = 0;
-	while (fscanf(refsfound, "%s%s%s", file, function, linenum) == 3) {
-		if ((i = strlen(pathcomponents(file, dispcomponents))) > filelen) {
-			filelen = i;
-		}
-		if (ogs == YES) {
-			ogsnames(file, &subsystem, &book);
-			if ((i = strlen(subsystem)) > subsystemlen) {
-				subsystemlen = i;
-			}
-			if ((i = strlen(book)) > booklen) {
-				booklen = i;
-			}
-		}
-		if ((i = strlen(function)) > fcnlen) {
-			fcnlen = i;
-		}
-		if ((i = strlen(linenum)) > numlen) {
-			numlen = i;
-		}
-		/* skip the line text */
-		while ((c = getc(refsfound)) != EOF && c != '\n') {
-			;
-		}
-		++totallines;
-	}
-	rewind(refsfound);
+	/* HBB 20041027: this used to hold a copy of the code of 
+	 * countrefs(), but with the crucial display width adjustments
+	 * missing.  Just call the real thing instead! */
+	countrefs();
 	return(YES);
 }
 
@@ -555,17 +524,17 @@ progress(char *what, long current, long max)
 			move(MSGLINE, 0);
 			clrtoeol();
 			addstr(what);
-			sprintf(msg, "%ld", current);
+			snprintf(msg, sizeof(msg), "%ld", current);
 			move(MSGLINE, (COLS / 2) - (strlen(msg) / 2));
 			addstr(msg);
-			sprintf(msg, "%ld", max);
+			snprintf(msg, sizeof(msg), "%ld", max);
 			move(MSGLINE, COLS - strlen(msg));
 			addstr(msg);
 			refresh();
 		}
 		else if (verbosemode == YES)
 		{
-			sprintf(msg, "> %s %ld of %ld", what, current, max);
+			snprintf(msg, sizeof(msg), "> %s %ld of %ld", what, current, max);
 		}
 
 		start = now;
@@ -603,7 +572,7 @@ myperror(char *text)
 		s = sys_errlist[errno];
 	}
 #endif
-	(void) sprintf(msg, "%s: %s", text, s);
+	(void) snprintf(msg, sizeof(msg), "%s: %s", text, s);
 	postmsg(msg);
 }
 
@@ -618,14 +587,24 @@ postmsg(char *msg)
 		fflush(stdout);
 	}
 	else {
-		move(MSGLINE, 0);
-		clrtoeol();
+		clearmsg();
 		addstr(msg);
 		refresh();
 	}
 	(void) strncpy(lastmsg, msg, sizeof(lastmsg) - 1);
 }
 
+/* clearmsg clears the first message line */
+
+void
+clearmsg(void)
+{
+	if (linemode == NO) {
+		move(MSGLINE, 0);
+		clrtoeol();
+	}
+}
+
 /* clearmsg2 clears the second message line */
 
 void
@@ -648,6 +627,7 @@ postmsg2(char *msg)
 	else {
 		clearmsg2();
 		addstr(msg);
+		refresh();
 	}
 }
 
@@ -663,23 +643,37 @@ posterr(char *msg, ...)
     {
         (void) vfprintf(stderr, msg, ap); 
 	(void) fputc('\n', stderr);
-    }
-    else
-    {
-#if HAVE_VSNPRINTF
+    } else {
         vsnprintf(errbuf, sizeof(errbuf), msg, ap);
-#else
-        vsprintf(errbuf, msg, ap);
-#endif
         postmsg2(errbuf); 
     }
 }
 
+/* display a fatal error mesg -- stderr *after* shutting down curses */
+void
+postfatal(const char *msg, ...)
+{
+	va_list ap;
+	char errbuf[MSGLEN];
+
+	va_start(ap, msg);
+	vsnprintf(errbuf, sizeof(errbuf), msg, ap);
+	/* restore the terminal to its original mode */
+	if (incurses == YES) {
+		exitcurses();
+	}
+
+	/* display fatal error messages */
+	fprintf(stderr,"%s",errbuf);
+
+	/* shut down */
+	myexit(1);
+}
 
 /* position references found file at specified line */
 
 void
-seekline(int line) 
+seekline(unsigned int line) 
 {
 	int	c;
 
