@@ -30,7 +30,7 @@
  DAMAGE. 
  =========================================================================*/
 
-/* $Id: scanner.h,v 1.3 2001/07/11 13:36:27 broeker Exp $ */
+/* $Id: scanner.h,v 1.5 2009/08/28 14:28:27 nhorman Exp $ */
 
 
 #ifndef CSCOPE_SCANNER_H
@@ -60,9 +60,10 @@
 #define UNIONDEF	'u'
 
 /* other scanner token types */
-#define LEXEOF	0
-#define	IDENT	1
-#define	NEWLINE	2
+#define	LEXEOF	0
+#define	LEXERR	1
+#define	IDENT	2	
+#define	NEWLINE	3	
 
 /* scanner.l global data */
 extern	int	first;		/* buffer index for first char of symbol */
@@ -76,7 +77,7 @@ extern	int	myylineno;	/* input line number */
 /* HBB 20010430: if lex is used instead of flex, have to simulate the
  * private copies of yytext and yytext for the world outside scanner.l: */
 /* FIXME: there should be a feature test for this! */
-#if defined(__OSF1__) || defined(__sun) 
+#if defined(__OSF1__) || defined(__sun) || defined(_AIX)
 extern	char	yytext[];
 #else
 extern	unsigned char	yytext[];
